/*
 * (C) Copyright 2000 Mulle kybernetiK
 *
 * Check out www.mulle-kybernetik.com for licensing details
 *
 * $Id$
 *
 * $Log$
 *
 */
#import "ImageInfoController.h"
#import "NSPrintInfo+ImagePrinterAdditions.h"
#import "NATUnitFormatter.h"
#import "NATControlBox.h"
#import "NSDecimalNumber+ImagePrinterAdditions.h"
#import "NATMoveAndSizeableRectView.h"


#define LS_CLOSING_WINDOW  NSLocalizedString( @"Closing window", "")
#define LS_TODO_VALUES     NSLocalizedString( @"What should be done with the values ?", "")
#define LS_USE             NSLocalizedString( @"Use", "")
#define LS_IGNORE          NSLocalizedString( @"Ignore", "")


#define HEIGHT_CHANGED  (1 << 5)
#define WIDTH_CHANGED   (1 << 4)
#define BOTTOM_CHANGED  (1 << 3)
#define RIGHT_CHANGED   (1 << 2)
#define TOP_CHANGED     (1 << 1)
#define LEFT_CHANGED    (1 << 0)
#ifdef ALL
#undef ALL
#endif
#define ALL      0xFFFFFF

#define getTop()      rectangle.origin.y
#define getBottom()   (rectangle.origin.y + rectangle.size.height - 1)
#define getLeft()     rectangle.origin.x
#define getRight()    (rectangle.origin.x + rectangle.size.width - 1)
#define getWidth()    rectangle.size.width
#define getHeight()   rectangle.size.height

#define setTop( dst, value)     ((dst).origin.y = (value))
#define setBottom( dst, value)  ((dst).origin.y = (value) - (dst).size.height + 1)
#define setLeft( dst, value)    ((dst).origin.x = (value))
#define setRight( dst, value)   ((dst).origin.x = (value) - (dst).size.width + 1)
#define setWidth( dst, value)   ((dst).size.width = (value))
#define setHeight( dst, value)  ((dst).size.height = (value))


//
// General Idea. User Paper Info as Application measure
// Calculate and Feedback everything in points.
//
NSString  *NATPrintInfoChanged = @"NATPrintInfoChanged";


@implementation ImageInfoController


- initWithImage:(NSImage *) p
        scaling:(NSImageScaling) scale
{
   NSRect  frame;
   
   [super initWithWindowNibName:@"ImageInfo"];

   image          = [p retain];

   frame.origin.x = 0;
   frame.origin.y = 0;
   frame.size     = [image size];
   
   imageView      = [[NATMoveAndSizeableRectView alloc] initWithFrame:frame];
   [imageView setDelegate:self];
   
   [imageView setImage:image];
   [imageView setImageScaling:NSScaleProportionally];

   rectangle = frame;
   stretch   = scale;
   
   [[NSNotificationCenter defaultCenter] addObserver:self
                                            selector:@selector( redrawWindow:)
                                                name:NATNotificationMeasurementUnitChanged
                                              object:nil];
   return( self);
}


- (void) dealloc
{
   [[NSNotificationCenter defaultCenter] removeObserver:self];

   [image release];
   [printInfo release];
   [delegate release];
   [super dealloc];
}


- (void) setDelegate:(id) p
{
   id   old = delegate;

   delegate = [p retain];   // not so hot really
   [old release];
}


- (id) delegate
{
   return( delegate);
}


- (void) setScaling:(int) value
{
   stretch = value;
}


- (BOOL) preserveRatio
{
   return( [lockButton state] == NSOnState);
}


- (unsigned int) setRectangle:(NSRect) wish
{
   unsigned int   mask = 0;
   float          aspect;
   NSSize         size;
   
   if( [self preserveRatio])
   {
      size   = [image size];
      aspect = size.height / size.width;
   
      wish.size.height = wish.size.width * aspect;
   }
   
   if( wish.size.width < 1.0)
   {
      wish.size.width = 1.0;
      mask |= WIDTH_CHANGED | RIGHT_CHANGED;
   }
   
   if( wish.size.height < 1.0)
   {
      wish.size.height = 1.0;
      mask |= HEIGHT_CHANGED | BOTTOM_CHANGED;
   }

   if( wish.size.width > useablePaperRect.size.width)
   {
      if( [self preserveRatio])
         return( ALL);
      wish.size.width = useablePaperRect.size.width;
      mask |= WIDTH_CHANGED | RIGHT_CHANGED;
   }
   
   if( wish.size.height > useablePaperRect.size.height)
   {
      if( [self preserveRatio])
         return( ALL);
      wish.size.height = useablePaperRect.size.height;
      mask |= HEIGHT_CHANGED | BOTTOM_CHANGED;
   }

   if( wish.origin.x < useablePaperRect.origin.x)
   {
      wish.origin.x = useablePaperRect.origin.x;
      mask |= LEFT_CHANGED | RIGHT_CHANGED;
   }
   if( wish.origin.y < useablePaperRect.origin.y)
   {
      wish.origin.y = useablePaperRect.origin.y;
      mask |= TOP_CHANGED | BOTTOM_CHANGED;
   }

   if( wish.origin.x + wish.size.width  > useablePaperRect.origin.x + useablePaperRect.size.width)
   {
      wish.origin.x = (useablePaperRect.size.width - wish.size.width) + useablePaperRect.origin.x;
      mask |= LEFT_CHANGED | RIGHT_CHANGED;
   }
   if( wish.origin.y + wish.size.height > useablePaperRect.origin.y + useablePaperRect.size.height)
   {
      wish.origin.y = (useablePaperRect.size.height - wish.size.height) + useablePaperRect.origin.y;
      mask |= TOP_CHANGED | BOTTOM_CHANGED;
   }

   mask |= ((rectangle.origin.x != wish.origin.x) << 0) |
           ((rectangle.origin.y != wish.origin.x) << 1) |
           (((rectangle.origin.x + rectangle.size.width)
            != (wish.origin.x + wish.size.width)) << 2)  |
           (((rectangle.origin.y + rectangle.size.height)
            != (wish.origin.y + wish.size.height)) << 3) |
           ((rectangle.size.width != wish.size.width) << 4)  |
           ((rectangle.size.height != wish.size.height) << 5);
   
   rectangle = wish;
   if( mask)
      [[self window] setDocumentEdited:YES];
   
   return( mask);
}


/*#
###
###
*/
- (void) setLeftUnits:(float) value
{
   NSRect   wish;

   wish = rectangle;

   setLeft( wish, value);
   [self visoSetRectangle:wish];
}


- (void) setTopUnits:(float) value
{
   NSRect   wish;

   wish = rectangle;
   setTop( wish, value);
   [self visoSetRectangle:wish];
}


- (void) setRightUnits:(float) value
{
   NSRect   wish;

   wish = rectangle;

   setRight( wish, value);
   [self visoSetRectangle:wish];
}


- (void) setBottomUnits:(float) value
{
   NSRect   wish;

   wish = rectangle;
   setBottom( wish, value);
   [self visoSetRectangle:wish];
}


- (void) setWidthUnits:(float) value
{
   NSRect   wish;

   wish = rectangle;
   if( [self preserveRatio])
      setHeight( wish, value * wish.size.height / wish.size.width);
   setWidth( wish, value);

   [self visoSetRectangle:wish];
}


- (void) setHeightUnits:(float) value
{
   NSRect   wish;

   wish = rectangle;
   if( [self preserveRatio])
      setWidth( wish, value * wish.size.width / wish.size.height);
   setHeight( wish, value);

   [self visoSetRectangle:wish];
}


//
// Text Field Handling
//
- (void) valueHasChanged:(id) sender
{
   if( sender == leftControl)
      [self setLeftUnits:[leftControl floatValue]];

   if( sender == topControl)
      [self setTopUnits:[topControl floatValue]];

   if( sender == rightControl)
      [self setRightUnits:[rightControl floatValue]];

   if( sender == bottomControl)
      [self setBottomUnits:[bottomControl floatValue]];

   if( sender == widthControl)
      [self setWidthUnits:[widthControl floatValue]];

   if( sender == heightControl)
      [self setHeightUnits:[heightControl floatValue]];
}



- (void) visoSetRectangle:(NSRect) wish
{
   unsigned int  mask;

   if( mask = [self setRectangle:wish])
   {
      [self syncImageView];
      [self syncControlsUsingMask:mask];
   }
}


- (void) scaleImageDownToFitKeepingAspect
{
   NSSize  size;
   float   hratio;
   float   vratio;
   NSRect  wish;

   wish = rectangle;
   size = useablePaperRect.size;
   
   NSAssert( size.width >= 1.0 && size.height >= 1.0, @"paper size is fucked up");

   hratio = size.width / rectangle.size.width;
   vratio = size.height / rectangle.size.height;

   if( hratio < 1.0 || vratio < 1.0)
   {
      if( hratio < vratio)
         vratio = hratio;

      wish.size.width  *= vratio;
      wish.size.height *= vratio;
   }

   if( wish.origin.x < useablePaperRect.origin.x)
      wish.origin.x = useablePaperRect.origin.x;
   if( wish.origin.y < useablePaperRect.origin.y)
      wish.origin.y = useablePaperRect.origin.y;
   
   [self setRectangle:wish];
}


/*#
###
###
*/
- (void) syncControlsUsingMask:(unsigned int) mask
{
   if( mask & LEFT_CHANGED)
      [leftControl setFloatValue:getLeft()];
   
   if( mask & TOP_CHANGED)
      [topControl setFloatValue:getTop()];
   
   if( mask & RIGHT_CHANGED)
      [rightControl setFloatValue:getRight()];
   
   if( mask & BOTTOM_CHANGED)
      [bottomControl setFloatValue:getBottom()];
   
   if( mask & WIDTH_CHANGED)
      [widthControl setFloatValue:getWidth()];
   
   if( mask & HEIGHT_CHANGED)
      [heightControl setFloatValue:getHeight()];
}


#warning (nat) hackish

- (void) syncControlsWithPaperSize
{
   NATUnitFormatter   *formatter;

   formatter = [[[NATUnitFormatter alloc] init] autorelease];
   [formatter setMinimum:[NSDecimalNumber decimalNumberWithFloat:0.0]];
   [formatter setMaximum:[NSDecimalNumber decimalNumberWithFloat:paperSize.width - 1.0]];
#if DEBUG   
   NSLog( @"Formatter: %@", formatter);
#endif   
   [leftControl setFormatter:formatter];
   [rightControl setFormatter:formatter];

   formatter = [[[NATUnitFormatter alloc] init] autorelease];
   [formatter setMinimum:[NSDecimalNumber decimalNumberWithFloat:0.0]];
   [formatter setMaximum:[NSDecimalNumber decimalNumberWithFloat:paperSize.height - 1.0]];
   [topControl setFormatter:formatter];
   [bottomControl setFormatter:formatter];

   formatter = [[[NATUnitFormatter alloc] init] autorelease];
   [formatter setMinimum:[NSDecimalNumber decimalNumberWithFloat:1.0]];
   [formatter setMaximum:[NSDecimalNumber decimalNumberWithFloat:paperSize.width]];
   [widthControl setFormatter:formatter];

   formatter = [[[NATUnitFormatter alloc] init] autorelease];
   [formatter setMinimum:[NSDecimalNumber decimalNumberWithFloat:1.0]];
   [formatter setMaximum:[NSDecimalNumber decimalNumberWithFloat:paperSize.height]];
   [heightControl setFormatter:formatter];
}



- (void) syncImageView
{
   [pageView setNeedsDisplayInRect:[pageView convertRect:[imageView frame]
                                                fromView:imageView]];
   [pageView setNeedsDisplay:YES]; // InRect:frame];
   [imageView setFrame:rectangle];
   [imageView setNeedsDisplay:YES];
}


//
// Calculate proper bounds value for aspect correct
// display

- (void) syncPageView
{
   NSRect   frame;
   NSSize   size;
   float    hratio;
   float    vratio;
   float    ratio;
   
   frame = [pageView frame];

   vratio = frame.size.height / paperSize.height;
   hratio = frame.size.width / paperSize.width;

   ratio = hratio;
   if( vratio < hratio)
      ratio = vratio;

//
// Now we know the scaling factor for an as big as possible
// aspect correct page view. Now we need to calculate how much
// we have to extend the bounds to cover the rest of the view   
//
   size.width = frame.size.width - (paperSize.width * ratio);
   size.height = frame.size.height - (paperSize.height * ratio);
//
// Size contains noe the leftover size in frame measurement
// calc it back to page units (points)
   size.width  = size.width / ratio + paperSize.width;
   size.height = size.height / ratio + paperSize.height;

   [pageView setPrintInfo:printInfo];
   [pageView setBoundsSize:size];
   [pageView setNeedsDisplay:YES];
}


- (void) syncWithPrintInfo
{
   paperSize        = [printInfo paperSize];
   useablePaperRect = [printInfo useablePaperRect];

   [self syncPageView];

   [self scaleImageDownToFitKeepingAspect];
   [self syncImageView];
   
   [self syncControlsWithPaperSize];
   [self syncControlsUsingMask:ALL];   
}


- (void) setImageInfo:(NSRect) rect
{
   rectangle = rect;
}


- (void) setPrintInfo:(NSPrintInfo *) p
{
   id   old = printInfo;

   printInfo = [p retain];
   [old release];

   [[NSNotificationCenter defaultCenter] addObserver:self
                                            selector:@selector( printInfoChanged:)
                                                name:NATPrintInfoChanged
                                              object:printInfo];
}


- (void) printInfoChanged:(NSNotification *) noification
{
   if( [noification object] == printInfo)
      [self syncWithPrintInfo];
}


- (void) showWindow:(id) sender
{
   [self window]; // force NIB file load
   [[self window] useOptimizedDrawing:YES];  // we aint overlapping anywhere
   
   [lockButton setState:stretch == NSScaleProportionally];
//   [pageBox addSelectableSubview:imageView];
   [self syncWithPrintInfo];
   [super showWindow:sender]; // loads NIB for real
}


//
// Some initial reconfiguration
//
- (void) awakeFromNib
{
   [pageView addSubview:imageView];
}


//
// Localize this
//
- (BOOL) windowShouldClose:(id) sender
{
   if( [[self window] isDocumentEdited])
   {
      if( NSRunAlertPanel( LS_CLOSING_WINDOW,
                           LS_TODO_VALUES,
                           LS_USE,
                           LS_IGNORE,
                           nil) == NSAlertDefaultReturn)
      {
         [delegate setImageInfo:rectangle];
      }
      [[self window] setDocumentEdited:NO];
   }
   return( YES);
}

@end
