/* ---------------------------------------------------------------------------
 *  $Id$
 *
 *  Copyright (c) 2000 by Mulle kybernetiK. All rights reserved.
 *
 *  Permission to use, copy, modify and distribute this software and its
 *  documentation is hereby granted, provided that both the copyright notice
 *  and this permission notice appear in all copies of the software,
 *  derivative works or modified versions, and any portions thereof, and that
 *  both notices appear in supporting documentation, and that credit is given
 *  to Mulle kybernetiK in all documents and publicity pertaining to direct or
 *  indirect use of this code or its derivatives.
 *
 *  THIS IS EXPERIMENTAL SOFTWARE AND IT IS KNOWN TO HAVE BUGS, SOME OF WHICH
 *  MAY HAVE SERIOUS CONSEQUENCES. THE COPYRIGHT HOLDER ALLOWS FREE USE OF
 *  THIS SOFTWARE IN ITS "AS IS" CONDITION. THE COPYRIGHT HOLDER DISCLAIMS ANY
 *  LIABILITY OF ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING DIRECTLY OR
 *  INDIRECTLY FROM THE USE OF THIS SOFTWARE OR OF ANY DERIVATIVE WORK.
 * ---------------------------------------------------------------------------
 *  $Log$
 * ---------------------------------------------------------------------------
 */
#import "ImagePrinterDocument.h"
#import "NATUnitFormatter.h"
#import "NSPrintInfo+NSNumberAdditions.h"
#import "NATPageLayout.h"
#import "PageLayoutAccessoryController.h"
#import "ImageInfoController.h"
#import "NATEmptyView.h"


#define LS_NOTING_TO_PRINT   NSLocalizedString( @"Nothing to print yet", "")
#define LS_CONTINUE          NSLocalizedString( @"Continue", "");


#define FILE_VERSION    0x0003
#define ENCODE_VERSION  1


@implementation ImagePrinterDocument

- (void) tryLoadDefault
{
   NSString   *path;
   NSData     *data;
   
   path = [[NSUserDefaults standardUserDefaults] stringForKey:@"LastFileName"];
   if( path && [path length])
   {
      if( data = [[[NSData alloc] initWithContentsOfFile:path] autorelease])
      if( [self loadDataRepresentation:data
                                ofType:nil])
         [self setFileName:path];  // no nee
   }
}


- init
{
   [super init];

   printInfo = [[NSPrintInfo sharedPrintInfo] copy];
   [printInfo setHorizontalPagination:NSClipPagination];
   [printInfo setVerticalPagination:NSClipPagination];
   [printInfo setHorizontallyCentered:NO];
   [printInfo setVerticallyCentered:NO];

   stretch = NSScaleProportionally;
   [[NSNotificationCenter defaultCenter] addObserver:self
                                            selector:@selector( redrawWindows:)
                                                name:NATNotificationMeasurementUnitChanged
                                              object:nil];
   [self tryLoadDefault];
   return( self);
}


- (void) dealloc
{
   [[NSNotificationCenter defaultCenter] removeObserver:self];
   [imageInfoController release];
   [printInfo release];
   [super dealloc];
}


- (NSString *) windowNibName
{
   return( @"ImagePrinterDocument");
}


- (void) _setImageInfo:(NSRect) rect
{
   imageInfo = rect;
}


//
// Reset ImageInfo size values to Image in ImageView
//
- (void) _useOriginalSize
{
   NSImage   *image;

   if( ! (image = [imageView image]))
   {
      image = [NSImage imageNamed:@"icon"];
      [imageView setImage:image];
   }

   imageInfo.size = [image size];
}


/*#
###
### Synchronize UI with our data
### 
#*/
- (void) syncViewsWithStretch
{
   [stretchPopUp selectItemAtIndex:[stretchPopUp indexOfItemWithTag:stretch]];
   [imageView setImageScaling:stretch];
}


- (void) syncCellsWithPrintInfo
{
   [pageBox setTitle:[NSString stringWithFormat:@"PaperSize %@", [printInfo paperName]]];
   
   [pagetopmarginCell setFloatValue:[printInfo topMargin]];
   [pageleftmarginCell setFloatValue:[printInfo leftMargin]];
   [pagerightmarginCell setFloatValue:[printInfo rightMargin]];
   [pagebottommarginCell setFloatValue:[printInfo bottomMargin]];
   
   [pagewidthCell setFloatValue:[printInfo paperSize].width];
   [pageheightCell setFloatValue:[printInfo paperSize].height];
}


- (void) syncCellsWithImageInfo
{
   [imagexCell setFloatValue:imageInfo.origin.x];
   [imageyCell setFloatValue:imageInfo.origin.y];
   [imagewidthCell setFloatValue:imageInfo.size.width];
   [imageheightCell setFloatValue:imageInfo.size.height];
}


- (void) sync
{
   [self syncCellsWithPrintInfo];
   [self syncCellsWithImageInfo];
   [self syncViewsWithStretch];
}


/*#
###
###
#*/ 
- (void) windowControllerDidLoadNib:(NSWindowController *) aController
{
   NATUnitFormatter   *formatter;
   
   [super windowControllerDidLoadNib:aController];

   // Add any code here that need to be executed once the windowController has loaded the document's window.
   formatter = [[[NATUnitFormatter alloc] init] autorelease];
   [imagexCell setFormatter:formatter];
   [imageyCell setFormatter:formatter];
   [imagewidthCell setFormatter:formatter];
   [imageheightCell setFormatter:formatter];
   
   [pagetopmarginCell setFormatter:formatter];
   [pageleftmarginCell setFormatter:formatter];
   [pagerightmarginCell setFormatter:formatter];
   [pagebottommarginCell setFormatter:formatter];
   
   [pagewidthCell setFormatter:formatter];
   [pageheightCell setFormatter:formatter];

   [[aController window] setMiniwindowImage:[NSImage imageNamed:@"icon.tiff"]];
}


/*#
###
### I/O Routines to custom ImagePrinterInfo file format
###
#*/
- (NSData *) dataRepresentationOfType:(NSString *)aType
{
   NSMutableData   *data;
   NSArchiver      *encoder;
   int             version = FILE_VERSION;
   
   data    = [NSMutableData data];
   encoder = [[[NSArchiver alloc] initForWritingWithMutableData:data] autorelease];
#if ENCODE_VERSION   
   [encoder encodeValueOfObjCType:@encode( int)
                               at:&version];
#endif
   [encoder encodeValueOfObjCType:@encode( int)
                               at:&stretch];
   [encoder encodeRect:imageInfo];
   [[printInfo dictionary] encodeWithCoder:encoder];  // MacOSX fucks up here #1#
   
   return( data);
}


- (BOOL) loadDataRepresentation:(NSData *) data
                         ofType:(NSString *) aType
{
   NSUnarchiver   *decoder;
   NSDictionary   *dictionary;
   int            version;
   
   decoder   = [[[NSUnarchiver alloc] initForReadingWithData:data] autorelease];
#if ENCODE_VERSION
   [decoder decodeValueOfObjCType:@encode( int)
                               at:&version];
   if( version != FILE_VERSION)
      return( NO);
#endif

   [decoder decodeValueOfObjCType:@encode( int)
                               at:&stretch];
   imageInfo  = [decoder decodeRect];
   [printInfo release];
   printInfo  = nil;
   dictionary = [[[NSDictionary alloc] initWithCoder:decoder] autorelease];
   printInfo  = [[NSPrintInfo alloc] initWithDictionary:dictionary];
   isLoaded   = YES;
   
   return( YES);
}


- (void) setFileName:(NSString *) s 
{
   [super setFileName:s];
   [[NSUserDefaults standardUserDefaults] setObject:s
                                             forKey:@"LastFileName"];
}


- (void) showWindows
{
   [super showWindows];
   if( ! isLoaded)
      [self _useOriginalSize];
   [self sync];
}
   

/*#
###
### Various action methods including printing
###
#*/
- (void) zeroMargins:(id) sender
{
   [printInfo setTopMargin:0];
   [printInfo setLeftMargin:0];
   [printInfo setRightMargin:0];
   [printInfo setBottomMargin:0];

   [self syncCellsWithPrintInfo];
   [[NSNotificationCenter defaultCenter] postNotificationName:NATPrintInfoChanged
                                                       object:printInfo
                                                     userInfo:nil];
}


//
// Open Image Placement Window
//
- (void) runImagePlacement:(id)sender
{
   if( ! imageInfoController)
   {
      imageInfoController = [[ImageInfoController alloc] initWithImage:[imageView image]
                                                               scaling:stretch];
      [imageInfoController setDelegate:self];
   }
   [imageInfoController setImageInfo:imageInfo];
   [imageInfoController setPrintInfo:printInfo];
   [imageInfoController setScaling:stretch];
   
   [imageInfoController showWindow:self];
}


typedef struct
{
   NATPageLayout                   *layouter;
   PageLayoutAccessoryController   *controller;
} AnotherLameAppleCallbackContext;


- (void) pageLayoutDidEnd:(NSPageLayout *) pageLayout
               returnCode:(int) returnCode
              contextInfo:(void *) contextInfo
{
   AnotherLameAppleCallbackContext  *info;

   info = (AnotherLameAppleCallbackContext *) [(NSData *) contextInfo bytes];
   
   if( returnCode == NSOKButton)
   {
#if DEBUG
      NSLog( @"after %@", printInfo);
#endif
      [self syncCellsWithPrintInfo];
      [[NSNotificationCenter defaultCenter] postNotificationName:NATPrintInfoChanged
                                                          object:printInfo
                                                        userInfo:nil];
   }
   
   [info->layouter release];
   [info->controller release];
   [(id) contextInfo release];
}


- (void) customRunPageLayout:(id)sender
{
   NSMutableData   *data;
   AnotherLameAppleCallbackContext  *info;
   
   data = [[NSMutableData alloc] initWithLength:sizeof( AnotherLameAppleCallbackContext)];
   info = (AnotherLameAppleCallbackContext *) [data mutableBytes];
   info->layouter   = [[NATPageLayout alloc] init];
   info->controller = [[PageLayoutAccessoryController alloc] init];

   [info->layouter setAccessoryView:[info->controller accessoryView]];
   [info->layouter setAccessoryViewController:info->controller];
   [info->controller setPageLayout:info->layouter];
   
#if DEBUG      
   NSLog( @"before %@", printInfo);
#endif
   [info->layouter beginSheetWithPrintInfo:printInfo
                            modalForWindow:[[[self windowControllers] lastObject] window]
                                  delegate:self
                            didEndSelector:@selector( pageLayoutDidEnd:returnCode:contextInfo:)
                               contextInfo:data];
}


- (void) useOriginalSize:(id) sender
{
   [self _useOriginalSize];
   [self syncCellsWithImageInfo];
}


- (void) imageStretchChosen:(id) sender
{
   stretch = [[sender selectedItem] tag];
   [imageView setImageScaling:stretch];
}


- (void) setImageInfo:(NSRect) rect
{
   [self _setImageInfo:rect];
   [self syncCellsWithImageInfo];
}


- (NSPrintInfo *) printInfo
{
   return( printInfo);
}


- (void) printView:(NSView *) view
{   
   if( ! [view window])
   {
      NSWindow  *window;

      window = [[[NSWindow alloc] initWithContentRect:[view frame]
                                            styleMask:NSBorderlessWindowMask
                                              backing:NSBackingStoreNonretained
                                                defer:YES] autorelease];
      [window setContentView:view];
   }
#if DEBUG
   NSLog( @"print %@", [self printInfo]);
#endif   
   [[NSPrintOperation printOperationWithView:view
                                   printInfo:[self printInfo]] runOperation];
}


/*#
### 
###
#*/ 
- (void) redrawWindows:(NSNotification *) notification
{
   [[self windowControllers] makeObjectsPerformSelector:@selector( redrawWindow:)
                                             withObject:notification];

}


- (void) printDocument:(id) sender
{
   NSRect            frame;
   NSView            *background;
   NSImageView       *printView;

   if( ! [imageView image])
   {
      NSRunAlertPanel( nil, @"Nothing to print yet", @"Continue", nil, nil);
      return;
   }

   printView  = [[[NSImageView alloc] initWithFrame:imageInfo] autorelease];
   [printView setImageFrameStyle:NSImageFrameNone];
   [printView setImageScaling:stretch];
   [printView setImage:[imageView image]];

   frame.origin.x = 0;
   frame.origin.y = 0;
   frame.size     = [printInfo paperSize];

   background = [[[NATEmptyView alloc] initWithFrame:frame] autorelease];
   [background addSubview:printView];

   [self printView:background];
}

@end

// #1#
//   NSPrintInfo just encodes the dictionary itself and so the
//   decode produces a dictionary not a NSPrintInfo
