/*
 * (C) Copyright 2000 Mulle kybernetiK
 *
 * Check out www.mulle-kybernetik.com for licensing details
 *
 * $Id$
 *
 * $Log$
 *
 */
#import "Defines.h"
#import "NATPageLayout.h"
#import "NATUnits.h"

#define USE_NAT_UNITS  1   // for reuse set to zero (or factor out another subclass)
#define SEND_PAGE_LAYOUT_CHANGED_NOTIFICATION 0  // 

NSString  *NATNotificationPageLayoutChanged = @"NATNotificationPageLayoutChanged";


@implementation NATPageLayout


+ (id) sharedInstance
{
   static id   sharedInstance;

   if( ! sharedInstance)
      sharedInstance = [[self alloc] init];
   return( sharedInstance);
}


- (void) dealloc
{
}


- (void) setAccessoryViewController:(id) controller
{
   accessoryViewController = controller;
}


- (id) accessoryViewController
{
   return( accessoryViewController);
}


//
//
//
- (float) factor
{
   float  factor;
   float  bogus;
   
   [self convertOldFactor:&bogus
                newFactor:&factor];
   return( factor);
}


- (void) sendLayoutChangedMessageForPrintInfo:(NSPrintInfo *) printInfo
                            originalPrintInfo:(NSPrintInfo *) memo
{
   if( [memo paperSize].width != [printInfo paperSize].width ||
       [memo paperSize].height != [printInfo paperSize].height ||
       [memo topMargin] != [printInfo topMargin] ||
       [memo leftMargin] != [printInfo leftMargin] ||
       [memo rightMargin] != [printInfo rightMargin] ||
       [memo bottomMargin] != [printInfo bottomMargin])
   {
      [[NSNotificationCenter defaultCenter] postNotificationName:NATNotificationPageLayoutChanged
                                                          object:printInfo
                                                        userInfo:nil];
   }
}


- (void) setUnitsWithNewFactor
{
   float   bogus;
   float   factor;
   int     unit;

   [self convertOldFactor:&bogus
                newFactor:&factor];
   unit = -1;
   if( factor == 1.0)
      unit = UnitPoints;
   if( factor >= 1 / 28.3 && factor <= 1 / 28.4)
      unit = UnitCM;
   if( factor == 1 / 72)
      unit = UnitInch;
   if( factor == 1 / 12)
      unit = UnitPica;
   if( unit != -1)
      [NATUnits setCurrentUnit:unit];
}


typedef struct
{
   id           delegate;
   SEL          didEndSelector;
   void         *contextInfo;
   NSPrintInfo  *memo;
} LameAppleCallbackContext;



- (void) pageLayoutDidEnd:(NSPageLayout *) pageLayout
               returnCode:(int) returnCode
              contextInfo:(void *) contextInfo
{
   LameAppleCallbackContext  *info;

   NSAssert( [(id) contextInfo isKindOfClass:[NSData class]], @"parameter mismatch");
   info = (LameAppleCallbackContext *) [(NSData *) contextInfo bytes];

   if( info->didEndSelector)
      objc_msgSend( info->delegate, info->didEndSelector, pageLayout, returnCode, info->contextInfo);
//      [info->delegate performSelector:info->didEndSelector
//                           withObject:(id) returnCode
//                           withObject:info->contextInfo];

   
#if USE_NAT_UNITS
   if( returnCode == NSOKButton)
      [self setUnitsWithNewFactor];
#endif
#if SEND_PAGE_LAYOUT_CHANGED_NOTIFICATION
   if( returnCode == NSOKButton)
      [self sendLayoutChangedMessageForPrintInfo:printInfo
                               originalPrintInfo:memo];
#endif
   [(id) info->contextInfo release];
   [info->memo release];
   [(id) contextInfo release];
}


- (void) beginSheetWithPrintInfo:(NSPrintInfo *) printInfo
                  modalForWindow:(NSWindow *) window
                        delegate:(id) delegate
                  didEndSelector:(SEL) didEndSelector
                     contextInfo:(void *) contextInfo 
{
   NSMutableData       *data;
   LameAppleCallbackContext   *info;
   
//   if( [accessoryViewController respondsToSelector:@selector( printInfoKnown:)])
//      [accessoryViewController printInfoKnown:printInfo];

   data = [[NSMutableData alloc] initWithLength:sizeof( LameAppleCallbackContext)];
   info = (LameAppleCallbackContext *) [data mutableBytes];
   info->delegate       = delegate;
   info->didEndSelector = didEndSelector;
   info->contextInfo    = [(id) contextInfo retain];
   info->memo           = [printInfo copy];  // not autoreleased!

   [super beginSheetWithPrintInfo:printInfo
                   modalForWindow:window
                         delegate:self
                   didEndSelector:@selector( pageLayoutDidEnd:returnCode:contextInfo:) // piss poor interface
                      contextInfo:data];
}

@end
