/*
 * (C) Copyright 2000 Mulle kybernetiK
 *
 * Check out www.mulle-kybernetik.com for licensing details
 *
 * $Id$
 *
 * $Log$
 *
 */
#import "NATPageView.h"


//
// A white square with a gray border
// the white square represents the
// printable region, the gray border the
// left over paper.
//
@implementation NATPageView


- (id) initWithFrame:(NSRect) frame
{
   [super initWithFrame:frame];

   printInfo = [[NSPrintInfo sharedPrintInfo] retain];
   paperSize = [printInfo paperSize];
   scale     = 1.0;
   autoFit   = YES;
   [self rebound];

   return( self);
}


//- (void) setScale:(float) value
//{
//   scale = value;
//}


- (float) scale
{
   return( scale);
}


//- (void) setAutoFit:(BOOL) flag
//{
//   autoFit = flag;
//}
//
//
- (BOOL) autoFit
{
   return( autoFit);
}


//
// should force redraw of view here I guess
//
- (void) setPrintInfo:(NSPrintInfo *) info
{
   id  old = printInfo;
   
   printInfo = [info retain];
   paperSize = [printInfo paperSize];
   
   [old release];
}


- (void) rebound
{
   NSRect  rect;
   
   rect = [self frame];
   
   if( autoFit)
   {
      float   hratio;
      float   vratio;
      
      hratio = paperSize.width  / rect.size.width;
      vratio = paperSize.height / rect.size.height;
      scale  = vratio < hratio ? hratio : vratio;
   }
   
   rect.size.width  *= scale;
   rect.size.height *= scale;

   [self setBoundsSize:rect.size];
}


//- (void) setScaleInPercent:(int) percent
//{
//   scale = 1.0 / (percent / 100.0);
//   [self rebound];
//}


- (void) setFrameSize:(NSSize) newSize
{
   [super setFrameSize:newSize];
   [self rebound];
}


- (void) setFrame:(NSRect) frameRect
{
   NSRect   rect;

   rect = [self frame];
   [super setFrame:frameRect];
   if( rect.size.width != frameRect.size.width ||
       rect.size.height != frameRect.size.height)
      [self rebound];
}


- (BOOL) isFlipped
{
   return( YES);
}


//
// non optimal method
// it would be better to calculate 4 margin rectangles
// and draw these and then the white paper rectangle
//
- (void) drawRect:(NSRect) rect
{
   NSRect  paper;
   NSRect  invertedMargin;
   NSRect  draw;
   
   paper.origin.x    = 0;
   paper.origin.y    = 0;
   paper.size.width  = paperSize.width;
   paper.size.height = paperSize.height;

   invertedMargin              = paper;
   invertedMargin.origin.x     = [printInfo leftMargin];
   invertedMargin.origin.y     = [printInfo topMargin];
   invertedMargin.size.width  -= invertedMargin.origin.x + [printInfo rightMargin];
   invertedMargin.size.height -= invertedMargin.origin.y + [printInfo bottomMargin];
   
   draw = NSIntersectionRect( paper, rect);
   [[NSColor lightGrayColor] set];
   NSRectFill( draw);

   draw = NSIntersectionRect( invertedMargin, rect);
   [[NSColor whiteColor] set];
   NSRectFill( draw);
}


- (NSSize) paperSize
{
   return( paperSize);
}


@end
