/*
 * (C) Copyright 2000 Mulle kybernetiK
 *
 * Check out www.mulle-kybernetik.com for licensing details
 *
 * $Id$
 *
 * $Log$
 *
 */
#import <Foundation/Foundation.h>
#import "NATUnitFormatter.h"

//
// Subclass of NSNumberformatter to adapt the formatted values
// to the currently active unit. This way we can internally always
// work with points, but give the user a different cultural view
//
@implementation NATUnitFormatter


- init
{
   NSDecimalNumberHandler  *rounder;
   
   [super init];
   [self setFormat:@"##0.###"];
   rounder = [NSDecimalNumberHandler decimalNumberHandlerWithRoundingMode:NSRoundPlain
                                                                    scale:3
                                                         raiseOnExactness:NO
                                                          raiseOnOverflow:NO
                                                         raiseOnUnderflow:NO
                                                      raiseOnDivideByZero:NO];
   [self setRoundingBehavior:rounder];
   return( self);
}



static inline double  convertToPoints( double x) 
{
   return( x / [NATUnits currentPointsFactor]);
}


static inline double  convertFromPoints( double  x) 
{
   return( x * [NATUnits currentPointsFactor]);
}


- stringForObjectValue:(id) anObject
{
   if( [anObject isKindOfClass:[NSNumber class]])
      anObject = [NSNumber numberWithFloat:convertFromPoints( [anObject floatValue])];
   return( [super stringForObjectValue:anObject]);
}


- (NSAttributedString *) attributedStringForObjectValue:(id) anObject
                                  withDefaultAttributes:(NSDictionary *) attributes
{
//   if( [anObject isKindOfClass:[NSNumber class]])
//      anObject = [NSNumber numberWithFloat:convertFromPoints( [anObject floatValue])];
//
#warning (nat) hack! we assume/know that NSNumberFormatter will call stringForObjectValue
   return( [super attributedStringForObjectValue:anObject
                           withDefaultAttributes:attributes]);
}


- (BOOL) getObjectValue:(id *) anObject
              forString:(NSString *) string
       errorDescription:(NSString **) error
{
   BOOL       flag;
   NSNumber   *foo;

   if( flag = [super getObjectValue:&foo
                          forString:string
                   errorDescription:error])
   {
      *anObject = [NSNumber numberWithFloat:convertToPoints( [foo floatValue])];
   }
   return( flag);
}

@end
