/*
 * (C) Copyright 2000 Mulle kybernetiK
 *
 * Check out www.mulle-kybernetik.com for licensing details
 *
 * $Id$
 *
 * $Log$
 *
 */
#import <Foundation/NSString.h>
#import "NATUnits.h"


NSString   *NATNotificationMeasurementUnitChanged = @"NATNotificationMeasurementUnitChanged";

//
// Centralized Unit handling
//
@implementation NATUnits

static NATUnitType    unit = -1;


+ (NATUnitType) currentUnit
{
   return( unit);
}


#warning (nat) oughta localize this
+ (NSString *) measurementString
{
   static NSString  *name[] =
   {
      @"Points",
      @"Centimeter",
      @"Inch",
      @"Pica",
   };
   return( name[ unit]);
}


+ (void) setCurrentUnit:(NATUnitType) value
{
   NSAssert1( value >= UnitPoints && value <= UnitPica, @"unknown unit %d", value);
   if( unit != value)
   {
      unit = value;
      [[NSUserDefaults standardUserDefaults] setObject:[self measurementString]
                                                forKey:@"NSMeasurementUnit"];
      [[NSNotificationCenter defaultCenter] postNotificationName:NATNotificationMeasurementUnitChanged
                                                          object:[NSNumber numberWithInt:unit]
                                                        userInfo:nil];
   }
}


+ (void) checkUserDefaults
{
   NSString   *unitName;

   if( unit != -1)
      return;
   
   unitName = [[NSUserDefaults standardUserDefaults] stringForKey:@"NSMeasurementUnit"];
   if( ! unitName)
   {
      unit = UnitPoints;
      return;
   }

   unitName = [unitName uppercaseString];
   
   if( [unitName hasPrefix:@"CEN"])
   {
      unit = UnitCM;
      return;
   }
   if( [unitName hasPrefix:@"PIC"])
   {
      unit = UnitPica;
      return;
   }
   if( [unitName hasPrefix:@"INC"])
   {
      unit = UnitInch;
      return;
   }
   unit = UnitPoints;
}


//
// Initialize and load clash! so we do the userDefaults check
// in load too...
//
+ (void) load
{
   NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];

   [self checkUserDefaults];
   [pool release];
}


+ (void) initialize
{
   [self checkUserDefaults];
}


+ (float) pointsGroup
{
   switch( unit)
   {
   case UnitPoints   :
      return( 100.0);

   case UnitCM     :
      return( 28.3465);

   case UnitInch   :
      return( 72.0);

   case UnitPica   :
      return( 12.0);
   }
#if DEBUG
   return( 1.0);
#else
   return( 0.0);
#endif   
}


+ (float) currentPointsFactor
{
   switch( unit)  
   {                  
   case UnitPoints   :
      return( 1.0);
      
   case UnitCM     :
      return( 1 / 28.3465);
             
   case UnitInch   :
      return( 1 / 72.0);
      
   case UnitPica   :
      return( 1 / 12.0);      
   }
#if DEBUG   
   return( 0.0);
#else
   return( 1.0);
#endif   
}


+ (float) pointsFactorForUnitType:(NATUnitType) type
{
   switch( type)  
   {                  
   case UnitPoints   :
      return( 1.0);
      
   case UnitCM     :
      return( 1 / 28.3465);
             
   case UnitInch   :
      return( 1 / 72.0);
      
   case UnitPica   :
      return( 1 / 12.0);
   }
#if DEBUG   
   return( 0.0);
#else
   return( 1.0);
#endif   
}

@end
