/*
 * (C) Copyright 2000 Mulle kybernetiK
 *
 * Check out www.mulle-kybernetik.com for licensing details
 *
 * $Id$
 *
 * $Log$
 *
 */
#import "Defines.h"
#import "PageLayoutAccessoryController.h"
#import "NATPageLayout.h"


@implementation PageLayoutAccessoryController


- init
{
   [super init];
   
   if( ! [NSBundle loadNibNamed:@"PageLayoutAccessory"
                          owner:self])
   {
      NSLog( @"PageLayoutAccessory load failed!!");
      [self release];
      return( nil);
   }
   
   return( self);
}


- (void) dealloc
{
   if( pageLayout)
      [[NSNotificationCenter defaultCenter] removeObserver:self];
   [pageLayout release];
   [super dealloc];
}


- (void) setPageLayout:(NSPageLayout *) p
{
   id                    old = pageLayout;
   NSNotificationCenter  *center;
   center = [NSNotificationCenter defaultCenter];
   
   if( old)
      [center removeObserver:self];
   [center addObserver:self
              selector:@selector( windowDidBecomeKey:)
                  name:NSWindowDidBecomeKeyNotification
                object:pageLayout];  // object never works..
      
   pageLayout = [p retain];
   [old release];
}


- (NSView *) accessoryView
{
   return( box);
}


- (void) redisplayWithPrintInfo:(NSPrintInfo *) printInfo
                         factor:(float) factor
{
   [topCell setFloatValue:[printInfo topMargin] * factor];
   [leftCell setFloatValue:[printInfo leftMargin] * factor];
   [rightCell setFloatValue:[printInfo rightMargin] * factor];
   [bottomCell setFloatValue:[printInfo bottomMargin] * factor];
}


- (void) redisplay
{
   [self redisplayWithPrintInfo:[pageLayout printInfo]
                         factor:[pageLayout factor]];  // hu hummm...
}


//
// Conceivably a category for NSPrintInfo!
//
- (void) sanifyMargins:(NSPrintInfo *) printInfo
              original:(NSPrintInfo *) original
{
   if( [printInfo topMargin] + [printInfo bottomMargin] <= [printInfo paperSize].height &&
       [printInfo leftMargin] + [printInfo rightMargin] <= [printInfo paperSize].width)
      return;

   if( original)
   {
      [printInfo setTopMargin:[original topMargin]];
      [printInfo setLeftMargin:[original leftMargin]];
      [printInfo setRightMargin:[original rightMargin]];
      [printInfo setBottomMargin:[original bottomMargin]];

      if( [printInfo topMargin] + [printInfo bottomMargin] <= [printInfo paperSize].height &&
          [printInfo leftMargin] + [printInfo rightMargin] <= [printInfo paperSize].width)
         return;
   }

   [printInfo setTopMargin:0.0];
   [printInfo setLeftMargin:0.0];
   [printInfo setRightMargin:0.0];
   [printInfo setBottomMargin:0.0];
}


//
//
// Text Field Handling
//
- (void) controlTextDidEndEditing:(NSNotification *) sender
{
   NSPrintInfo  *printInfo;
   NSPrintInfo  *clone;
   NSForm       *form;
   NSCell       *cell;
   float        factor;
   float        value;

   form      = [sender object];
   cell      = [form cellAtIndex:[form indexOfSelectedItem]];
   NSAssert( [cell isKindOfClass:[NSFormCell class]], @"you done wrong, son!");
   
   printInfo = [pageLayout printInfo];
   factor    = [pageLayout factor];
   
   value  = [cell floatValue] / factor;
   clone  = [[printInfo copy] autorelease];

   if( cell == topCell)
      [printInfo setTopMargin:value];

   if( cell == leftCell)
      [printInfo setLeftMargin:value];

   if( cell == rightCell)
      [printInfo setRightMargin:value];

   if( cell == bottomCell)
      [printInfo setBottomMargin:value];
   
   [self sanifyMargins:printInfo
              original:clone];
   [self redisplay]; 
}


//
// Messages from NATPageLayout
//
- (void) pickedOrientation:(id)sender
{
   [self sanifyMargins:[pageLayout printInfo]
              original:nil];
   [self redisplay];
}


- (void) pickedPaperSize:(id)sender
{
   [self sanifyMargins:[pageLayout printInfo]
              original:nil];
   [self redisplay];
}


- (void) pickedUnits:(id)sender
{
   [self redisplay];
}


//- (void) printInfoKnown:(NSPrintInfo *) printInfo
//{
//   [self redisplayPrintInfo:printInfo
//                     factor:1.0];  // ???
//}


//
// Notification we are observing...
//
- (void)  windowDidBecomeKey:(NSNotification *) notification
{
   [self redisplay];
}

@end
